module Tablets
  # Incapsulates tablet related information
  class Base
    def self.inherited(child_class)
      tablet_name = child_class.tablet_name
      Tablets.register_tablet(tablet_name, child_class)
    end

    def self.tablet_name
      name.underscore.sub(/_tablet$/, '')
    end

    def name
      self.class.tablet_name
    end

    # Returns general jquery-datatable configuration overrides
    def options
      {}
    end

    # Determines is user authorized
    def authorize(controller)
      true
    end

    # Allows to make additional processing before records would be used
    def process(records)
      records
    end

    # Returns details HTML for the record
    def details(record)
      nil
    end

    # Returns columns definitions
    def columns
      deduct_columns_from_relation
    end

    # Returns database relation to fetch data
    def relation(options = {})
      name.camelize.constantize.where(nil)
    end

    # Returns additional data returned to client
    def payload
      {}
    end

    private

    # Returns default columns definititions, deducted from relation
    def deduct_columns_from_relation
      relation.columns.map(&:name).map do |name|
        {
          title: name.humanize.capitalize,
          data: name.to_sym,
          order: name
        }
      end
    end
  end
end
