module Tablets
  class Data
    module Processing
      # Base class for relation processings
      class Base
        # Initializes processor with relation and data reqired for processing
        def initialize(params, columns)
          @params = params
          @columns = columns
        end

        # Applies processing on relation. Need to be implemented in descendants
        def apply(_relation)
          fail NotImplementedError, '#apply need to be overrided by processing.'
        end

        # Shorthand for create processing and apply it
        def self.apply(params, columns, relation)
          new(params, columns).apply(relation)
        end

        private

        attr_reader :params, :columns
      end
    end
  end
end
