require 'active_support/core_ext/object/blank'

require 'tablets/utils/search_condition_builder'

require 'tablets/data/processing/base'

module Tablets
  class Data
    module Processing
      # Incapsulate relation filtering logic
      class Filter < Tablets::Data::Processing::Base
        # Applies filter processing on relation
        def apply(relation)
          search(relation)
        end

        private

        # Applies search conditions if need
        def search(relation)
          return relation unless params[:search].fetch(:value, nil).present?

          conditions = build_conditions_for(params[:search][:value])
          relation = relation.where(conditions) if conditions
          relation
        end

        # Builds search conditions
        def build_conditions_for(query)
          query.split(' ').map do |value|
            '(' + search_conditions(value) + ')'
          end.join(' AND ')
        end

        # Returns searchable columns
        def searchable_columns
          columns.map { |column| column[:search] }.flatten.compact.map(&:to_s)
        end

        # Returs search conditions for each searchable column
        def search_conditions(value)
          searchable_columns.map do |column|
            Tablets::Utils::SearchConditionBuilder.new(column, value).build
          end.join(' OR ')
        end
      end
    end
  end
end
