require 'tablets/data/processing/base'

module Tablets
  class Data
    module Processing
      # Incapsulate relation ordering logic
      class Order < Tablets::Data::Processing::Base
        # Applies order processing on relation
        def apply(relation)
          params[:order].values.inject(relation) do |rel, item|
            sorting_direction = direction(item)
            nulls = null_place(sorting_direction)
            rel.order("#{column(item)} #{sorting_direction} #{nulls}")
          end
        end

        private

        # Determines order column from params
        def column(item)
          columns[item[:column].to_i][:order]
        end

        # Determines order direction from params (ASC by default)
        def direction(item)
          if %w(ASC DESC).include?(item.fetch(:dir, '').upcase)
            item[:dir].upcase
          else
            'ASC'
          end
        end

        # Determines where nulls appear
        def null_place(sort)
          if sort.upcase == 'ASC'
            'NULLS FIRST'
          else
            'NULLS LAST'
          end
        end
      end
    end
  end
end
