require 'tablets/data/processing/base'

module Tablets
  class Data
    module Processing
      # Incapsulate relation pagination logic
      class Paginate < Tablets::Data::Processing::Base
        # Applies :start and :length from params to relation as offset and limit
        # :length is optional. Default value is used if no length is provided
        # :start recalculated to match beginnning of page
        def apply(relation)
          relation.offset(offset).limit(per_page)
        end

        private

        # Calculates offset as beginning of page.
        def offset
          (page - 1) * per_page
        end

        # Calculates page from start.
        def page
          (params[:start].to_i / per_page) + 1
        end

        # Returns default length.
        def default_length
          10
        end

        # Returns length or default value.
        def per_page
          params.fetch(:length, default_length).to_i
        end
      end
    end
  end
end
