require 'active_support/core_ext/object/blank'

require 'tablets/data/processing/filter'
require 'tablets/data/processing/paginate'
require 'tablets/data/processing/order'

module Tablets
  class Data
    # Incapsulates database query
    class Query
      # Initializes query with relation, params and columns
      def initialize(relation, params, columns)
        @relation = relation
        @params = params
        @columns = columns
      end

      # Applies all processings on relation and returns it
      def fetch
        paginate filter order relation
      end

      # Returns total records count before filter and pagination is applied
      def total
        relation.count(:all)
      end

      # Returns records count after filter is applied but before pagination
      def filtered
        filter(relation).count(:all)
      end

      private

      attr_reader :relation, :params, :columns

      # Applies order processing
      def order(records)
        return records unless params[:order].present?

        Tablets::Data::Processing::Order.apply(params, columns, records)
      end

      # Applies filter processing
      def filter(records)
        return records unless params[:search].present?

        Tablets::Data::Processing::Filter.apply(params, columns, records)
      end

      # Applies paginate processing
      def paginate(records)
        return records if params[:length] == '-1'

        Tablets::Data::Processing::Paginate.apply(params, columns, records)
      end
    end
  end
end
