require 'rails'

module Tablets
  module Global
    # Responsible to make application configurable
    module Configurator
      attr_accessor :options
      attr_accessor :tablets_dir

      # Setup configuration. Used in initializers
      def setup
        yield self
      end

      # Initialize default config values
      def self.extended(base)
        base.options = DEFAULT_OPTIONS
        base.tablets_dir = File.expand_path('app/tablets', Rails.root)
      end

      DEFAULT_OPTIONS = {
        pageLength: 15,
        processing: false,
        info: true,
        lengthChange: false,
        serverSide: true,
        pagingType: 'full_numbers',
        order: [0, 'asc'],
        autoWidth: true,
        orderCellsTop: true,
        serverMethod: 'POST',
        language: {
          lengthMenu: '<span class=\'seperator\'>|</span>View _MENU_ records',
          info: '<span class=\'seperator\'>|</span>Found total _TOTAL_ records',
          infoEmpty: '',
          emptyTable: 'No records found to show',
          zeroRecords: 'No matching records found',
          paginate: {
            previous: 'Prev',
            next: 'Next',
            last: 'Last',
            first: 'First',
            page: 'Page',
            pageOf: 'of'
          }
        }
      }
    end
  end
end
