require 'rails/railtie'

require 'tablets/view_helpers'

module Tablets
  # Rails hooks.
  class Railtie < ::Rails::Railtie
    # Allows tablets view helper to be used anywhere in rails views
    initializer 'tablets.view_helpers' do
      ActionView::Base.send :include, Tablets::ViewHelpers
    end

    reloader_class =
      if Rails::VERSION::MAJOR >= 5
        ActiveSupport::Reloader
      else
        ActionDispatch::Reloader
      end

    # Manages tablets loading and cleanup
    # Also ensures tablets reloading in development environment
    initializer 'tablets.reloader' do |app|
      reloader_class.to_prepare { Tablets.load! }
    end
  end
end
